# cel notes

separate packages out into feeds and blobs:

an package's metadata.json + tarball @ a specific version: BLOB

a signed feed that maps a (package, version) to a blob: FEED


# publish a package 2.0

1. create a non-live hyperdrive archive of package+version tarball + host

# thoughts

is the idea that every individual can act like a repository? everyone can
satisfy a request for a specific (package, version, hash) regardless of where
they got it from

this means that a user can import their local npm into peer-npm and treat those
packages the same as anything else in peer-npm

so, users then discover (package, version) hashes by following different users'
feeds. a feed is basically an alt-registry: a thing you follow that tells you
about new (package, version, hash) tuples

a question is: how do you deal with the fact that, in `package.json`, the key of
a dependency implicitly refers to the registry its name is in the context of?
the npm registry is identified by a package name *sans* any other information,
where as, like it is today, a network name and public key otherwise identify the
place the package comes from.

it sounds like the key change would be changing tarball references from requests
vs a public key to a request for a static hash.

wait, so what are the tarballs named / get written to metadata.json?
- write `foo-1.2.3.tgz` to hyperdrive archive HASH
- when writing metadata, write `HASH` as tarball filename
- users now ask for tarballs by `HASH`


# post-shower thoughts

two types of drivers:

1. tarball blob store
2. registry feed

## npm-tarball-store

### hash-blob-store

consumes a blob store: https://github.com/maxogden/abstract-blob-store

module that wraps it so it does
1. only SHA hashes as keys
2. checks the SHA hash after downloading

then people can store blobs on ipfs, hyperdrive, webtorrent, ssb, etc

## registry feed

anything that can satisfy:

1. getMetadata(pkgName) => JSON
2. publish(pkgName, tarball)

```js
var store = require('ssb-npm-tarball-store')(ssb)
var registry = SsbRegistry(ssb, store)

registry.get('through2', function (err, metadata) {
  // regular metadata json /w all known versions + their 'shasum's
})

registry.publish(uploadJson, function (err) {
})
```

OR: must a registry be a crypto-signed append-only log of `(package, version,
shasum)` entries?

# msg to cel

@cel What you wrote was extremely helpful! I'm reapproaching some aspects of the
project now in accordance to that.

Instead of having single type of peer network "driver", there will be two types
of drivers: a *blob driver* and a *registry driver*. All blobs (tarballs) are
referred to by their SHAsum (shame that npm uses SHA1..). This means that any
module that implements
[abstract-blob-store](https://github.com/maxogden/abstract-blob-store) can store
tarballs, which decouples node package tarballs from the registry it came from.
That's *so* powerful! Really glad you noticed that @cel.

The other driver is a *registry driver*, which anything that satisfies the interface

```
get(packageName, packageVersion, expectedShasum, function (err, packageMetadata) {})

publish(rawPackageMetadataFromNpmPublishCommand, blobStore, function (err) {})
```

In terms of storage, a registry then is just a big list of tuples of the form
`(packageName, version, shasum)`, and is decoupled fully from whatever blob
store that retrieves the actual tarball. Such a registry could be anything from
a hypercore feed, to an ssb materialized view, to a text file on your local
filesystem.


## q: how do you denote registry in swarmDeps?

same as current. "network" now only refers to registry.

what if it was instead:

```
"swarmDependencies": {
  "hyperdrive://field-trip_PUBKEY": "^0.5.1"
}
```

then maybe do git-style alt subcommands, so ^^^ assumes a `nps-hyperdrive`
subcommand

then that subcommand gets called:

```
nps-hyperdrive metadata field-trip_PUBKEY

# and

nps-hyperdrive publish field-trip_PUBKEY
```

so a user could publish to a specific registry using..

```
nps publish hyperdrive

# which will call

nps-hyperdrive publish  # with metadata + attachments its stdin
```

means npm itself is valid:

```
nps publish npm
```

^ which just does a vanilla `npm publish`

I think it'll be easiest to hardcode the blob store to just be hypercore +
discovery-swarm for now, with it all modular underneath, but expose that to the
CLI in the future


## todo

- make `hyperdrive.js` into just a registry
- make ^ accept an abstract-blob-store
- commands: `peer-npm` and `nps`
  - ^ think on this some more

## usage 2.0

what things do I want to make flow nicely now?

- hypercore-blob-store default + discovery-swarm
- some way to add your npm cache to your store
- some way to publish your local blob store as a registry via DHT/LAN
  - core idea: every `nps` user is their own mini p2p npm registry of their
    local npm package cache
- download the BIG db from npmjs.org and have a local registry you can use

## nps-blob-store

write-blob: writes tarball to local fs
read-blob: checks local fs; if not, finds peers discovery-swarm and downloads

## questions

- can I make your local npm cache your primary/only package DB?
  - then `nps daemon` can scan your cache and track all (package, version,
    shasum) values
- how does `nps install` work?
  - load in swarmDeps
  - use the registry listed to get the metadata for the package
  - use the hypercore-blob-store to locate and download tarballs
- is there any way I can do this without swarmDeps and deps rewriting?
  - you'd need to use the same registry for ALL deps, recursively
  - OR accept breaking vanilla `npm`
- how can swarmDeps work recursively?
  - 
- who gets to choose the registry you use? the package author or the consumer?
  - author: writes `PKG_NETWORK_KEY` to `swarmDependencies`
    + more honest representation of what the author intended
    - consumer may not have access to the registries the author used
  - consumer: maintains a list of registries they wish to use
- some way to specify just a shasum as a swarm dep, to skip registry stuff?
  - a shasum by itself should be sufficient

## registry styles

- web of trust
  - public key of a friend => use their npm cache
  - `PKG_hyperdrive_KEY`
- offline copy
  - download big package db from npm => your local machine is a registry
  - `PKG_cache`
- high trust community
  - ssb (all), ssb (specific feeds)
  - `PKG_ssb`, `PKG_ssb_FEEDID`
- centralized
  - npm
    - `PKG_npm`
  - you / your company runs a peer with a hypercore feed
    - `PKG_server_DOMAIN`


## play time: user managed registries

```
$ nps registry add noffle hyperdrive://7bc0bd9bd557547b81d53196674c869c6d698164c68b0033fd4b48849ce59110

$ nps registry add offline file:///home/noffle/packages.json

$ nps registry add cel ssb://@f/6sQ6d2CMxRUhLpspgGIulDxDCwYD7DzFzPNr7u5AU=.ed25519

$ nps registry add official npm://offical

$ nps registry add corp https://www.my-corp.com/registry
```

then

```
$ nps install -S foobar

WARNING: 'foobar@7.9.1' was found in multiple registries under different SHA1
hashes. Which registry's checksum would you like to use?
1) noffle  [SHASUM]
2) cel     [SHASUM]
Enter 'q' to quit: [q] _
```

also, syncing:

```
$ nps registry sync noffle

# or just

$ nps registry sync

# ^ to sync them all

$ nps registry sync official
Now syncing ALL packages from npmjs.org..
```

an author can include registries in their published `package.json`:

Q: does the `sync` logic imply that only a local registry is actually used?
like, you keep a local registry on your machine from the union of your `sync`s,
and so an `install` actually only needs to seek shasums?
A: that sounds nice.

```
$ nps publish --includeRegistries noffle,cel,official
```

then, a subsequent `nps install` will attempt to use these registries first.
this might mean `sync`ing them before use, if you do not already have them in
your registry list


## concerns

this approach loses namespaces: all node packages exist virtually across all
registries

this makes impersonation easier: publish a malicious package with the same name
as a popular non-malicious package to some registries that don't already contain
it

this is the advantage to package names that bake in their registry: they can be
trusted, if the registry given is used

can this new model capture that option?

disadvantage of baking in registry+key: you can no longer install this package unless you are using that specific registry. the trade-off is between security and package availability. I suspect different users will make different choices along this spectrum. Thus, `peer-npm` ought to do its best to permit both.


## registry baking / registry agnostic

If you wish to use a package in an agnostic manner, simply putting it by its simple-name in `"dependencies"` is sufficient. The user can then set up their desired registries and the priority order, and an `nps install` will retrieve it. Similarly, `nps install -S foobar` will place the new package in `"dependencies"`.

If registry bake-in is desired, the package should include a registry name and (optionally) a public key in hexadecimal. An entry like `"foobar_webtorrent_PUBKEY"` will appear in `"swarmDependencies"`. Similarly, one can write `nps install -S foobar_webtorrent_PUBKEY` to use that specific registry, and encode that fact in the `package.json`.

Q: How does recursive installation of packages with `swarmDependencies` work?
A: At publish-time, `nps` would need to publish the entire dependency tree to the swarm, not unlike npm shrinkwrapping.

Q: Registry bake-in, approached this way, seems complex and limited. Is there another way to get the security that a package referred to by a package is the one that ultimately gets installed?
 - Would allowing a package to specify a `trustedRegistries` list in `package.json` be helpful to this end?
 - The peer-npm server intercepts all package metadata, so it can rewrite the `package.json` of anything in the dependency tree before `npm` gets its hands on it. This seems like it ought to be enough..
 - What if instead of registry-baking, the package author could include the expected public key of each/any of its dependencies?
 

## public key hints on dependencies

```
"dependencies": {
  "foobar": "~1.0.0"
},
"dependencyKeys": {
  "foobar": "7bc0bd9bd557547b81d53196674c869c6d698164c68b0033fd4b48849ce59110"
}
```

In general, each peer-npm user could have a keypair they would consider their identity in the swarm.

Q: Where does the signature live? What does it sign?
Q: Does this imply that trusting one dep => its entire deptree is trusted?

More thinking on this!


## metadata json or just shasums?

What are the trade-offs for having a registry serve *just* the version SHA1-sums versus serving the entire package metadata JSON?

### just shasums
- registries are maximally simple / stupid: very good, security-wise
### all package metadata
- two sources of truth: registry and json inside tarball
- registry can fake metadata
- author could update metadata, like dist-tags


## problem: npm wants dependencies in metadata for dep-gathering

I think I need to grab the shasum, untar, look at the inner package.json, then grab its dependencies listing


## how should the default blob store work?

background
- use discovery-swarm, with a common key to link up all peers
- peers share their shasums on connect; blob-store remembers associations
- index + backed by everything in your npm cache?

READ
- pick a peer that has the tarball and request+stream it
- fetching a tarball should be over an encrypted channel
- make sure to check the SHA1 hash upon completion (`hashpipe` module?)

WRITE
- on blob WRITE, write meta + tgz to local npm cache


## how should the default p2p registry work?

- each peer has a hyperdrive describing the modules they have to share
- it should reflect what's in their npm cache
- `nps registry sync` will sync the two
- hyperdrive has `foo.json` which is that package's metadata
- also has `HASH.tgz` files for all tarballs it has
- users can whitelist hyperdrive pubkeys that it trusts


## flow: installing a module

```
nps daemon &
```

1. start daemon
2. make self available on discovery-swarm via the `peer-npm` key

```
nps registry sync
```

3. sync down all [trusted] peers' registry hyperdrives to local leveldb
4. update local index of `PKG => [METADATA]` mapping

```
nps install foobar
```

5. use local index of metadata info
6. for a wanted shasum, check local cache
7. if not in cache, fire up a `discovery-swarm` for the desired shasum(s)
8. download tarball from a peer & check hash


## Are there any downsides to only doing registry queries offline?

e.g. the `hyperdrive-registry` module would, on `sync()`, pull `PKG => METADATA` mappings down from peers and store them in a local leveldb

from here, all `fetchMetadata` queries would actually be against that big local db

one implication is that sync-able registries only need to impl a much simpler function:

```
var rs = registry.sync(new Date())
rs.on('data', function (pkg, metadata) {
  leveldb.put(pkg, {
    from: peerId,
    metadata: metadata
  })
})
```

I think this will make behaviour more predictable: you know you have registry entries for everything that you've synced.


## Scenarios I want to work well/easily

### 1. LAN cache share

Two or more friends on a LAN /w no or slow internet. They want to be able to do an `nps install` that uses the union of their package metadata and tarballs, all over multicast DNS, sans configuration.

### 2. Whitelist internet cache share

Same as LAN share, but a set of friends over the internet, with pre-known public keys, want to have a union of metadata / tarballs, and do installs/publishes.

### 3. Local full registry + swarm for tarballs

I want to sync the entire npm registry's package metadata to my laptop (sans tarballs), and be able to do a `git fetch`-like periodic delta sync that pulls down the latest package metadata from "the source" (npmjs.org).

From here, I want to be able to do a `nps install foobar` and have it use a hyper{drive,core} swarm to fetch the needed tarballs. The p2p network is trustless here: treated like a content-addressable store.

### 4. Vanilla npm use

As a fall-back.


## Capturing these scenarios

What would be MOST flexible would be letting the user specify git-like "remotes" for package metadata and for tarballs.

metadata: local-cache, http-registry, friend-pubkey
tarballs: local-cache, http-registry, p2p-swarm



## Example Session

```
$ npm install -g nps
...

$ nps 
```
