# what if peer-npm was about creating a decentralized npm future?

Like, what if peer-npm was about exploring the answer to the question: what if
*instead* of npm Inc., we *just* had a decentralized p2p world of npm packages?


## readmes

people could write

```
To install, just grab `peer-npm` and run:

    peer-npm install -g foobar_97b4ff0c6da6c0856302a4de235a7647
```

## publish

"publishing to the swarm" means recursively publishing all packages in the
dependency tree so that they are also in the swarm

- swarm deps in the tree do not need to be republished
- a package is ready to be published once all of its deps have "swarmDeps"
  equivalents


## install

you can install by a package's `name_pubkey`. this means there can be many
packaged versions of a package.

however, tarballs are content-addressed, meaning even if Alice and Bob publish
their respective versionf of `foobar@1.0.0` to `foobar_97b4ff0` and
`foobar_6dffd3` respectively, they *should* both point to the tarball
`6c0856f302a4de.tar.gz`.

`peer-npm install foobar_PUBKEY` will search for the package across

1. your local hyperdrive
2. over all peer-npm peers via mDNS
3. over all peer-npm peers via bittorrent/dht

the package metadata will be retrieved, and then the tarball will be downloaded
using the same process


# names

- peer-npm
- dex-npm
- dnpm
- friendpm
- swarm-npm

# goals

+ no centralization
+ packages are immutable & around forever (can't be deleted)
+ install packages over LAN
+ install packages over internet /wo central source
+ install packages locally while offline


# "PRODUCT" ANALYSIS

## Why am I building this thing?

I want npm to work offline or in wacky situations where I have no access to or
no longer want to use npmjs.org.

I want node packages to flow freely, without a central body. P2P tech has gotten
good enough that we can manage our own packages without corporate oversight.

The ideal is to work with npm Inc., to turn their registry into a high-bandwidth
high-availability peer in the network for these packages. I want to benefit npm
AND the community: the community gets npm access in a far more open and robust
way, and npm Inc gets reduced network costs, since users are sharing packages
amongst themselves than relying solely on a single data source.


## What is the problem I’m trying to solve?

One or more of the following is true:

a. offline
b. on low-bandwidth/metered-bandwidth
c. npmjs.org is down
d. not wanting to setup an npmjs.org account

and wish to install or share npm packages.


## Who has this problem?

1. Anyone who doesn't have an unfiltered high-bandwidth internet connection at
   all times.

2. Anyone who wants to use or develop npm packages who doesn't trust or wish to
   rely on a centralized for-profit american company for node/js development.


## Is this problem real?

1. Yes. People who are traveling, living offgrid or in a rural or
   non-first-world location. People who are in cafes or at conferences or at
   airports or on a tethered hotspot where internet is shoddy or metered.

2. Yes. The US and foreign governments are quite interested in monitoring and
   censoring. Having a centralized middleman at the mercy of government orders
   is a real risk.


## Has this problem been solved?

Let's look around..

- https://github.com/VictorBjelkholm/cowpen
  + probably the closest to what I want
  + missing: named (non-pinned hash) packages
  + pain: requires go-ipfs and FUSE to be installed system-wide + configured
  + goal: reproducible builds
  + goal: packages are around forever (can't be deleted)
  + goal: no centralization
  + goal: installing packages over LAN
  + goal: works offline via local IPFS cache
  + requires IPFS
  + requires FUSE
  + cool: it doesn't require a new CLI for installing; uses FUSE and npm quirks cleverly
  + however, does require cowpen CLI for publishing
- https://github.com/elsehow/gx-js
  + requires IPFS/go-ipfs
  + requires gx (written in Go)
  + unreleased (no install hash to install from!)
- https://github.com/dominictarr/npmd
  + hard to understand (docs are unclear/confusing; great proof that good docs make a HUGE difference)
  + seems to be angled toward making a strong local cache, not p2p use
- https://github.com/diasdavid/npm-on-ipfs
  + aims to mirror the official npm registry
  + use IPFS underneath for discovery+retrieval
- https://github.com/automenta/npn
  + readme looks like a big list of "future" features
  + no description of what it is/how it works
- https://github.com/nginnever/ippm
  + requires IPFS
  + wants to use Ethereum for trust/named repos
  + cool future goal: encrypted packages for private use
  + not super clear
  + seems very under-construction, not ready for use
- https://github.com/noffle/peer-npm
  + almost exactly what I want (surprise surprise :P)
  + could use even more clear/concise docs & examples
  + NEEDS recursive swarm publishing to be useful
  + WANT: version is derived from counting major/minor/patch releases, not explicit


## Can I solve it better, for instance by reducing friction?

I think so!

1. Not requiring anything to be installed other than `peer-npm` itself. No IPFS,
   no FUSE, etc.

2. Being a one-liner installation:
   - `npm install -g peer-npm`
   - (or, have a slightly more complicated process to install it from the swarm directly)

3. Have named repos / human-readable names (no illegible hashes).

4. Permit use of npm-style semver without hash-hunting.

5. Make sharing your package via the swarm as easy as a README note:

```
To install via peer-npm:

    peer-npm install foobar_97b4ff0c6da6c0856302a4de235a7647
```

BONUS: that swarm package name will never change! you can use that as a key to
download *any* version of `foobar` over the swarm.

6. Being simple to use: just write `peer-npm` in place of `npm` anywhere you'd
   `install` or `publish`, and it'll Just Work(tm)


## Is a software solution really all that is needed?

I think so!


## When you’ve identified the problem you’re trying to solve, the goals the users will have, ask yourself the following:

### How will they fulfill a specific goal in your software

1. "I want to publish my existing package 'to the swarm'"
2. "I want to publish somebody else's package to the swarm, for swarm use.
3. "I want to use npm over my local network with friends."


### what does a user’s journey through your software look like?


# "marketing"

- "like npm, but you don't need to make an npmjs.org account and you can install
  from your local machine, peers on the local network, and across the internet"

- "anyone can publish any package, no censorship, no name collisions, no
  gatekeepers"

- use the word "immutable" -- means people can't delete packages underneath you,
  or modify what's been published
  - make it clear that using `foobar_PUBKEY@3.0.0` is immutable: multiple version
    publishing is impossible

