# peer-npm

works just like npm, but:

- grabs tarballs from LAN and DHT or duplex
- can use npm registry or "the swarm" registry


# web o trust

what if web of trust was used?

- `npm install foobar` searches DHT+LAN for `foobar` that's been signed by a
  direct friend (can specify a `---trust-depth` if you want > 1)
- `peer-npm publish` publishes the current package version and all of its deps
  to the swarm, signed by the current user


# "remotes" model

- registry remotes
- tarball remotes

# original stated goals

1. The official npm package registry should remain a valid, legitimate registry, but no longer be *required* for node package management.
2. Users of peer-npm form a swarm: a peer-to-peer network where node packages are shared communally without a centralized source.
3. Anyone can use their machine's local npm package cache to serve packages to the rest of the swarm.
4. Users can install node packages over P2P networks, over local networks, and offline from their local machine.
5. peer-npm is readily hackable, so that adding new ways to share packages is easy. (Bluetooth? Sound waves? USB key sneakernets?)
6. Using peer-npm with your packages won't break your work: vanilla `npm` will keep on working.


# updated stated goals

less focus on decentralized idealogy, more focused on practicality of sharing
packages amongst each other without high-bandwidth internet

npm covers sharing packages when high-bandwidth internet is available. peer-npm
wants to cover all of the other situations people find themselves in.

1. robust: pluggable discovery and transports for every unconventional situation
2. respectful: works alongside vanilla npm; peer-npm packages aren't locked into continued peer-npm use
3. ready-to-rumble: easy to bootstrap a peer-to-peer install to get nearby friends using peer-npm


1. There should be many different ways to get packages, not just a central location that requires high-bandwidth internet as its gate
2. Users of peer-npm form a swarm: a peer-to-peer network where node packages are shared communally without a centralized source.
3. Anyone can use their machine's local npm package cache to serve packages to the rest of the swarm.
4. Users can install node packages over P2P networks, over local networks, and offline from their local machine.
5. peer-npm is transport agnostic, so that adding new ways to share packages is easy. (Bluetooth? Sound waves? USB key sneakernets?)
6. Using peer-npm with your packages won't break vanilla `npm`: packages published with peer-npm aren't locked in


# The 'git serve' of npm

1. use it to share a specific package (and its deps at your latest local)

2. use it to share a whole bunch of packages (a directory, your whole npm cache)


# Lots of angles

1. sharing packages in a one-off manner with people in the same room
2. treating the union of all discoverable peers as a global npm registry


---

NOTION: every peer is their own little npm registry!

NOTION: this should be the npm equivalent of 'git serve'!

---


# Why am I building this thing?

I want npm to work offline or in wacky situations where I have no access to or
no longer want to use npmjs.org.

_or_: I want to share packages in a one-off manner efficiently and securely with
people in the same room or area as me.


# What is the problem I’m trying to solve?

I'm offline or on low-bandwidth/metered-bandwidth and wish to install or share
npm packages.


# Who has this problem?

1. Anyone who doesn't have an unfiltered high-bandwidth internet connection at
   all times.

2. Anyone who wants to use or develop npm packages who doesn't trust or wish to
   rely on a centralized for-profit american company for node/js development.


# Is this problem real?

1. Yes. People who are traveling, living offgrid or in a rural or
   non-first-world location. People who are in cafes or at conferences or at
   airports or on a tethered hotspot where internet is shoddy or metered.

2. Yes. The US and foreign governments are quite interested in monitoring and
   censoring. Having a centralized middleman at the mercy of government orders
   is a real risk.


# Has this problem been solved?

Good question! I should do some research on that.

I *think* there are some modules for syncing or sharing your local npm cache,
but that might be it? I'm not aware of anything that is explicitly peer-to-peer.

# Can I solve it better, for instance by reducing friction?

I think so!

1. Being transport-agnostic. DHTs, LANs, sneakernets, or pluggable duplex
   streams for discovery and replication. This makes it useful in a more varied
   and unpredictable diversiy of situations.

2. Being very easy to install and use.
```
   <LAPTOP 1>
   $ npm install -g peer-npm

   $ cd great-package
   $ peer-npm publish great-package
   (published great-package@3.0.0 and all dependencies at local versions)

   $ peer-npm share

   <LAPTOP 2>
   $ npm install -g peer-npm

   $ cd my-app
   $ peer-npm install -S great-package

   6ffb5aeee54da02896f0d8585e8ef (cel) is sharing great-package@3.0.0
   Trust this peer and install [(y)es  (j)ust this one time  (n)o]  y<CR>

   Give this peer name [cel]: <CR>
   Added 6ffb5aeee54da02896f0d8585e8ef as a trusted peer named 'cel'

   Installed: great-package@3.0.0
```

# Is a software solution really all that is needed?

I think so!

# When you’ve identified the problem you’re trying to solve, the goals the users will have, ask yourself the following:
## How will they fulfill a specific goal in your software
## what does a user’s journey through your software look like?

- user stories
- mockup sessions

> The purpose of all of these exercises is to make sure that you’re not wasting
> time and money building the wrong thing. It is expensive to build something,
> and you want to know that you’re on the right track before you start pouring
> resources into software development.

> Spending time on design, testing, validation may seem like a waste of time
> when it seems clear to you what you need to build, but in reality it can save
> you time by quickly identifying where to focus your limited resources. It is
> much faster to change a prototype and test it than to change your entire
> product once you’ve put a lot of resources into building it.


# User Stories

1. Leyla, the Conference Speaker
 - share her cool new module + peer-npm itself over local network for mass installation

2. Natalie, the Nomad
 + natalie is always on the move, which means lots of cafe and airport wifi:
   metered and slow
 + she travels with different friends, who also hack on node/js
3. Lucius, the Offgrid/Rural Hacker
 + lucius lives in a solar cabin and has limited 3G internet each month
 + his friends come and visit here and there, who also hack on node/js
 - together, they have a much larger collection of npm modules than alone
 - they can do `peer-npm install` to share what they have with each other
 - they can collaborate on projects too! one person can `peer-npm publish` and
   the others can `peer-npm install` the latest version of their in-development
   module

4. Kara, the Hackerspace Dweller
 - shares her vast local npm cache of modules with hacker friends that come/go
 - sets up a raspberry pi that holds 10,000+ modules that any hackers can
   install without needing to wait on or saturate the internet


## How will they fulfill a specific goal in your software
## what does a user’s journey through your software look like?

1. Leyla, the Conference Speaker (and her eager audience)

```
Hey y'all, let's do a walkthrough of 'regl' together! The internet here is very
slow, and I bet none of you have heard of peer-npm, so just install it like
this:

Everyone@conf$ npm install -S regl --registry http://192.168.0.123:8000

_or_

Everyone@conf$ npm install -g peer-npm --registry http://192.168.0.123:8000
then
Everyone@conf$ peer-npm install -S regl

Before saying this, Leyla runs locally:

Leyla@conf$ peer-npm publish regl
Published regl@4.3.0

Leyla@conf$ peer-npm share
http://192.168.0.123:8000
```

2. Natalie, the Nomad (and her friend, Fauna)

```
N: Drat, this airport wifi is slow! It'll take forever for me to install lodash!

F: Oh, I think I have this package locally.

F@airport$ peer-npm share
Now sharing all npm packages in /home/fauna/.npm

F: Try now.

N@airport$ peer-npm install -S lodash

N: Thanks!
```

3. Lucius, the Cafe Coder (and his friend, Mel)

```
M: Okay, I'm finished working on those changes to the character recognition
   module we talked about.

L: Great! I'm ready to integrate it. Could you publish it?

M@cafe$ cd ~/dev/ocr-scanner
M@cafe$ peer-npm publish
Published ocr-scanner@4.1.0

M: Ready!

L@cafe$ cd ~/projects/book-scanner
L@cafe$ peer-npm upgrade ocr-scanner
Installed ocr-scanner@4.1.0

L: Thanks! Now I can test out these changes in the app!
```

4. Sammy, the Offgrid/Rural Hacker (and her friend, Hannes)

```
H: 9pm already? Sun's down. Shoot -- the solar battery is all used up and the
   wifi is dead. I didn't finish installing my dependencies for this project
   from npm.

S: That's okay, I installed them earlier today.

H: Alas! We have no local wifi for us to share with!

S: Pff, no problem. You can install those deps from my laptop over bluetooth!

S@cabin$: peer-npm share -t bluetooth
Bluetooth Code: 7438

H@cabin$: peer-npm install -t bluetooth
Bluetooth Code? 7438
Installed all dependencies!

_or_ via a USB key (sneakernet)

S plugs in a USB key
S@cabin$: cd ~/work/solar-display
S@cabin$: peer-npm share . -t file > /media/usb/deps

S hands USB key to H, H plugs it into his laptop
H@cabin$: peer-npm install -t file < /media/usb/deps
Installed all dependencies!
```

# What features were used above?

1. Host your npm cache as an npm registry over local network via http
2. Publish a package to your local cache
3. Host your npm cache as an npm registry discoverable+downloadable over local network
4. Install packages from any other peer-npm user on the local network who is sharing
5. Host your npm cache as an npm registry over bluetooth
6. Host a subset of your npm cache by writing them to a USB stick
7. Install packages from an archive on a USB stick
8. Specify an external shell command to use to perform replication


# Flow chart

1. Do we have high-bandwidth uncensored internet?

Yes: use npm

No: goto 2

2. Do all users have peer-npm installed?

No: - run `peer-npm share --bootstrap` and have others use `npm install -g peer-npm --registry=ADDR`
    - goto 3

Yes: goto 3

3. 
